# -*- coding:utf-8 -*-
# Create your views here.
from coupon_manager.forms import *
from coupon_manager.models import Vale
from parque_vehicular.models import Vehiculo
from django.db.models import Sum
from django.shortcuts import render_to_response
from django.http import HttpResponseRedirect, HttpResponse
from django.template import *
from reportlab.pdfgen import canvas
from reportlab.lib.pagesizes import letter, landscape
from reportlab.lib.units import cm, mm, inch
from reportlab.lib.enums import TA_JUSTIFY, TA_RIGHT, TA_LEFT, TA_CENTER
from reportlab.platypus import Table, TableStyle, SimpleDocTemplate, PageTemplate, Paragraph, Spacer, Image
from reportlab.lib import colors
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from django.db import IntegrityError, transaction
from django.contrib import messages

def bulk_create_vales(request):
	if request.method == 'POST':
		form = AgregaVales(request.POST)
		if form.is_valid():
			no_serie_inicial = form.cleaned_data['no_serie_inicial']
			no_serie_final = form.cleaned_data['no_serie_final']
			monto = form.cleaned_data['monto']
			proveedor = form.cleaned_data['proveedor']
			mes_asignacion = int(form.cleaned_data['mes_asignacion'])
			anio_asignacion = form.cleaned_data['anio_asignacion']

			request.session['no_serie_inicial'] = no_serie_inicial
			request.session['no_serie_final'] = no_serie_final
			request.session['proveedor'] = proveedor
			
			# Crear vales
			vales = []
			for no_serie in xrange(no_serie_inicial,no_serie_final + 1):
				vale = Vale(no_serie=no_serie, monto=monto, proveedor=proveedor)
				vales.append(vale)

			try:
				vales_creados = Vale.objects.bulk_create(vales)

				# Crear un arreglo de todos los lunes del mes para asignar los vales.
				import calendar
				cal = calendar.Calendar()
				iter_mes = cal.itermonthdates(anio_asignacion, mes_asignacion)
				luneses = []
				for f in iter_mes:
					if f.month == mes_asignacion and calendar.weekday(f.year, f.month, f.day) == 0:
						luneses.append(f)

				# Asignar vales a vehiculos activos por semana
				vehiculos_activos = Vehiculo.objects.filter(activo=True)
				for lunes in luneses:
					for vehiculo in vehiculos_activos:
						if len(Vale.objects.filter(no_serie__range=(no_serie_inicial,no_serie_final), vehiculo=None)) == 0:
							break
						# si el vehiculo en esta fecha no ha recibido suficientes vales para completar su consumo minimo, asignarle hasta que cumpla la cuota
						monto_asignado = Vale.objects.filter(vehiculo=vehiculo,fecha_asignacion=lunes).aggregate(Sum('monto'))['monto__sum']
						monto_asignado = monto_asignado if monto_asignado != None else 0
						if monto_asignado < vehiculo.tipo.consumo_semanal or monto_asignado == 0:
							monto_acumulado = monto_asignado
							while monto_acumulado < vehiculo.tipo.consumo_semanal and len(Vale.objects.filter(no_serie__range=(no_serie_inicial,no_serie_final), vehiculo=None)) > 0:
								vale = Vale.objects.filter(no_serie__range=(no_serie_inicial,no_serie_final), vehiculo=None)[0]
								vale.vehiculo = vehiculo
								vale.fecha_asignacion = lunes
								vale.save()
								monto_acumulado += vale.monto
	
				# Asignar vales aleatoriamente si aun hay sin asignar
				vales_sin_asignacion = Vale.objects.filter(no_serie__range=(no_serie_inicial, no_serie_final), vehiculo=None)
				if len(vales_sin_asignacion) > 0:
					import random
					for vale in vales_sin_asignacion:
						vehiculos = Vehiculo.objects.filter(activo=True)
						vehiculo = vehiculos[random.randint(0,vehiculos.count() - 1)]
						fecha = luneses[random.randint(0,len(luneses) - 1 )]
						vale.fecha_asignacion = fecha
						vale.vehiculo = vehiculo
						vale.save()

				msj = '$' + str(len(vales_creados) * monto) + ' en vales creados y asignados correctamente'
				messages.success(request, msj)

			except IntegrityError as _error:
				transaction.rollback()
				msj = _error
				messages.error(request, msj)

			return HttpResponseRedirect('/admin/')
	else:
		form = AgregaVales()
	return render_to_response('vales_form.html', {'form' : form}, context_instance=RequestContext(request))

def informe(request, anio, mes):
	if mes == '1':
		mes_humano = 'Enero'
	elif mes == '2':
		mes_humano = 'Febrero'
	elif mes == '3':
		mes_humano = 'Marzo'
	elif mes == '4':
		mes_humano = 'Abril'
	elif mes == '5':
		mes_humano = 'Mayo'
	elif mes == '6':
		mes_humano = 'Junio'
	elif mes == '7':
		mes_humano = 'Julio'
	elif mes == '8':
		mes_humano = 'Agosto'
	elif mes == '9':
		mes_humano = 'Septiembre'
	elif mes == '10':
		mes_humano = 'Octubre'
	elif mes == '11':
		mes_humano = 'Noviembre'
	else:
		mes_humano = 'Diciembre'

	vales = Vale.objects.filter(fecha_asignacion__month=mes, fecha_asignacion__year=anio)
	monto_por_proveedor = vales.values('proveedor__nombre').annotate(suma=Sum('monto'))
	monto_total = vales.aggregate(Sum('monto'))['monto__sum']
	vehiculos = Vehiculo.objects.filter(id__in=vales.values('vehiculo').distinct())
	return render_to_response('informe.html', {'vehiculos' : vehiculos, 'anio' : anio, 'mes' : mes, 'mes_humano' : mes_humano, 'monto_total' : monto_total, 'monto_por_proveedor' : monto_por_proveedor})

def reporte(request, anio, mes, vehiculo_id):
	response = HttpResponse(mimetype='application/pdf')
	estilo=getSampleStyleSheet()
	elements = []
	doc = SimpleDocTemplate(response,pagesize=landscape(letter), rightMargin=5*mm,leftMargin=5*mm, topMargin=7*cm,bottomMargin=3*cm)
	data=[[
	'Fecha',
	'Importe',
	'Vale Inicial',
	'Vale Final',
	'Nombre y Firma']]

	vehiculo = Vehiculo.objects.get(pk=vehiculo_id)
	vales = Vale.objects.filter(vehiculo=vehiculo, fecha_asignacion__month=mes, fecha_asignacion__year=anio)
	monto_total = vales.aggregate(Sum('monto'))['monto__sum']
	fechas = vales.values('fecha_asignacion').distinct()
	rh = [1.5*cm]
	for f in fechas:
		vales_fecha = vales.filter(fecha_asignacion=f['fecha_asignacion'])

		generador = min_max(vales_fecha)
		for g in generador.__iter__():
			v = vales_fecha.filter(no_serie__range=(g))
			monto_vales = v.aggregate(Sum('monto'))['monto__sum']
			data.append([f['fecha_asignacion'],'$' + str(monto_vales), g[0], g[1], ''])
			rh.append(1.5*cm)

	data.append(['TOTAL','$' + str(monto_total), '', '', ''])

	rh.append(1.5*cm)

	t_datos = Table(data,
    	[4*cm, 4*cm, 4*cm, 4*cm, 10*cm],
    	rowHeights=rh,
    	repeatRows=1,
    	style=[
    	('GRID',(0,0),(-1,-1),0.5,colors.grey),
    	('FONTSIZE',(0,1),(-1,-1),10),
    	('FONTSIZE', (0,0),(-1,0), 14),
    	('FONTSIZE',(0,-1),(1,-1), 18),
    	('ALIGN',(0,0),(-1,-1),'CENTER'),
    	
    	('VALIGN',(0,0),(-1,-1),'MIDDLE'),
    	])

	t_datos.hAlign = 'CENTER'

	elements.append(t_datos)

	pageinfo = ''
	
	import os
	imagen = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'static/images/logo_sa.png')

	def myFirstPage(canvas, doc):
		canvas.saveState()
		canvas.drawImage(imagen,1*cm,510, 219, 90)

		canvas.setFont('Helvetica',12)
		canvas.drawString(3*cm, 17*cm, 'Marca: ' + vehiculo.tipo.marca.nombre)
		canvas.drawString(3*cm, 16.5*cm, 'Modelo: ' + vehiculo.tipo.tipo)
		canvas.drawString(3*cm, 16*cm, 'Año: ' + str(vehiculo.modelo))
		canvas.drawString(13*cm, 17*cm, 'Color: ' + vehiculo.color)
		canvas.drawString(13*cm, 16.5*cm, 'Placas: ' + vehiculo.placas)
		canvas.drawString(13*cm, 16*cm, 'Numero de Inventario: ' + vehiculo.no_inventario)

		canvas.restoreState()

	def myLaterPages(canvas, doc):
		canvas.saveState()
		canvas.drawImage(imagen,1*cm,510, 219, 90)

		canvas.setFont('Helvetica',12)
		canvas.drawString(3*cm, 17*cm, 'Marca: ' + vehiculo.tipo.marca.nombre)
		canvas.drawString(3*cm, 16.5*cm, 'Modelo: ' + vehiculo.tipo.tipo)
		canvas.drawString(3*cm, 16*cm, 'Año: ' + str(vehiculo.modelo))
		canvas.drawString(13*cm, 17*cm, 'Color: ' + vehiculo.color)
		canvas.drawString(13*cm, 16.5*cm, 'Placas: ' + vehiculo.placas)
		canvas.drawString(13*cm, 16*cm, 'Numero de Inventario: ' + vehiculo.no_inventario)

		canvas.restoreState()

	doc.build(elements, onFirstPage = myFirstPage, onLaterPages = myLaterPages)
	return response

def min_max(vales):
	vales = vales.order_by('no_serie')
	inicial = 0
	final = 0

	for v in vales:
		a = v.no_serie
		if final != 0 and a != final + 1:
			yield inicial, final
			inicial = a

		if inicial == 0:
			inicial = a

		final = a

	yield inicial, final
