# -*- coding:UTF-8 -*-
from django.db import models

# Create your models here.
class Marca(models.Model):
    nombre = models.CharField(max_length=50, unique=True)

    def __unicode__(self):
        return self.nombre

class Tipo(models.Model):
    tipo = models.CharField(max_length=255)
    marca = models.ForeignKey('Marca')
    consumo_semanal = models.DecimalField(max_digits=8, decimal_places=4,
        help_text=
            "Consumo semanal de gasolina (en pesos) aprox. para el vehiculo. Ingrese un valor múltiplo de 100")

    def __unicode__(self):
        return u'%s %s' % (self.tipo, self.marca)

    class Meta:
        unique_together = ('tipo', 'marca')
        ordering = ('tipo',)

class Vehiculo(models.Model):
    tipo = models.ForeignKey(Tipo)
    no_inventario = models.CharField(max_length=50,
        verbose_name="numero de inventario")
    no_serie = models.CharField(max_length=255,
        verbose_name='numero de serie', blank=True, null=True)
    no_motor = models.CharField(max_length=255,
        verbose_name='numero de motor', blank=True, null=True)
    placas = models.CharField(max_length=10)
    color = models.CharField(max_length=50, blank=True, null=True)
    modelo = models.PositiveIntegerField(blank=True, null=True)
    observaciones = models.TextField(blank=True, null=True)
    activo = models.BooleanField(default=True)

    def __unicode__(self):
        return u'%s %s %s' % (self.no_inventario, self.tipo, self.modelo)
